@@ -1,4 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 2 -*- */
+/* vim: set sw=2 et: */
 /* pager object */
 
 /*
@@ -22,6 +23,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
+#include <math.h>
+#include <glib/gi18n-lib.h>
+
 #include "pager.h"
 #include "workspace.h"
 #include "window.h"
@@ -30,7 +36,28 @@
 #include "pager-accessible-factory.h"
 #include "workspace-accessible-factory.h"
 #include "private.h"
-#include <math.h>
+
+/**
+ * SECTION:pager
+ * @short_description: a pager widget, showing the content of workspaces.
+ * @see_also: #WnckScreen
+ * @stability: Unstable
+ *
+ * A #WnckPager shows a miniature view of the workspaces, representing managed
+ * windows by small rectangles, and allows the user to initiate various window
+ * manager actions by manipulating these representations. The #WnckPager offers
+ * ways to move windows between workspaces and to change the current workspace.
+ *
+ * Alternatively, a #WnckPager can be configured to only show the names of the
+ * workspace instead of their contents.
+ *
+ * The #WnckPager is also responsible for setting the layout of the workspaces.
+ * Since only one application can be responsible for setting the layout on a
+ * screen, the #WnckPager automatically tries to obtain the manager selection
+ * for the screen and only sets the layout if it owns the manager selection.
+ * See wnck_pager_set_orientation() and wnck_pager_set_n_rows() to change the
+ * layout.
+ */
 
 #define N_SCREEN_CONNECTIONS 11
 
@@ -62,6 +89,9 @@ struct _WnckPagerPrivate
   guint dnd_time; /* time of last event during dnd (for delayed workspace activation) */
 };
 
+G_DEFINE_TYPE (WnckPager, wnck_pager, GTK_TYPE_WIDGET);
+#define WNCK_PAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), WNCK_TYPE_PAGER, WnckPagerPrivate))
+
 enum
 {
   dummy, /* remove this when you add more signals */
@@ -124,23 +154,28 @@ static gboolean wnck_pager_button_release (GtkWidget        *widget,
                                            GdkEventButton   *event);
 static gboolean wnck_pager_focus         (GtkWidget        *widget,
                                           GtkDirectionType  direction);
+static gboolean wnck_pager_query_tooltip (GtkWidget  *widget,
+                                          gint        x,
+                                          gint        y,
+                                          gboolean    keyboard_tip,
+                                          GtkTooltip *tooltip);
 static void workspace_name_changed_callback (WnckWorkspace *workspace,
                                              gpointer       data);
 
 static gboolean wnck_pager_window_state_is_relevant (int state);
-static gint wnck_pager_window_get_workspace   (WnckWindow  *window);
+static gint wnck_pager_window_get_workspace   (WnckWindow  *window,
+                                               gboolean     is_state_relevant);
 static void wnck_pager_queue_draw_workspace   (WnckPager   *pager,
 					       gint	    i);
 static void wnck_pager_queue_draw_window (WnckPager	   *pager,
 					  WnckWindow	   *window);
 
-static void wnck_pager_connect_screen    (WnckPager  *pager,
-                                          WnckScreen *screen);
+static void wnck_pager_connect_screen    (WnckPager  *pager);
 static void wnck_pager_connect_window    (WnckPager  *pager,
                                           WnckWindow *window);
 static void wnck_pager_disconnect_screen (WnckPager  *pager);
 
-static void wnck_pager_set_layout_hint   (WnckPager  *pager);
+static gboolean wnck_pager_set_layout_hint (WnckPager  *pager);
 
 static void wnck_pager_clear_drag (WnckPager *pager);
 static void wnck_pager_check_prelight (WnckPager *pager,
@@ -155,57 +190,43 @@ static GdkPixbuf* wnck_pager_get_background (WnckPager *pager,
 static AtkObject* wnck_pager_get_accessible (GtkWidget *widget);
 
 
-static gpointer parent_class;
-
-GType
-wnck_pager_get_type (void)
-{
-  static GType object_type = 0;
-
-  g_type_init ();
-  
-  if (!object_type)
-    {
-      const GTypeInfo object_info =
-      {
-        sizeof (WnckPagerClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) wnck_pager_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (WnckPager),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) wnck_pager_init,
-        NULL            /* value_table */
-      };
-      
-      object_type = g_type_register_static (GTK_TYPE_WIDGET,
-                                            "WnckPager",
-                                            &object_info, 0);
-    }
-  
-  return object_type;
-}
-
 static void
 wnck_pager_init (WnckPager *pager)
 {  
+  int i;
   static const GtkTargetEntry targets[] = {
     { "application/x-wnck-window-id", 0, 0}
   };
 
-  pager->priv = g_new0 (WnckPagerPrivate, 1);
+  pager->priv = WNCK_PAGER_GET_PRIVATE (pager);
 
   pager->priv->n_rows = 1;
   pager->priv->display_mode = WNCK_PAGER_DISPLAY_CONTENT;
   pager->priv->show_all_workspaces = TRUE;
   pager->priv->shadow_type = GTK_SHADOW_NONE;
+
   pager->priv->orientation = GTK_ORIENTATION_HORIZONTAL;
   pager->priv->workspace_size = 48;
-  pager->priv->bg_cache = NULL;
-  pager->priv->layout_manager_token = WNCK_NO_MANAGER_TOKEN;
+
+  for (i = 0; i < N_SCREEN_CONNECTIONS; i++)
+    pager->priv->screen_connections[i] = 0;
+
   pager->priv->prelight = -1;
+  pager->priv->prelight_dnd = FALSE;
+
+  pager->priv->dragging = FALSE;
+  pager->priv->drag_start_x = 0;
+  pager->priv->drag_start_y = 0;
+  pager->priv->drag_window = NULL;
+
+  pager->priv->bg_cache = NULL;
+
+  pager->priv->layout_manager_token = WNCK_NO_MANAGER_TOKEN;
+
+  pager->priv->dnd_activate = 0;
+  pager->priv->dnd_time = 0;
+
+  g_object_set (pager, "has-tooltip", TRUE, NULL);
 
   gtk_drag_dest_set (GTK_WIDGET (pager), 0, targets, G_N_ELEMENTS (targets), GDK_ACTION_MOVE);
   GTK_WIDGET_SET_FLAGS (GTK_WIDGET (pager), GTK_CAN_FOCUS);
@@ -216,9 +237,9 @@ wnck_pager_class_init (WnckPagerClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
-  
-  parent_class = g_type_class_peek_parent (klass);
-  
+
+  g_type_class_add_private (klass, sizeof (WnckPagerPrivate));
+
   object_class->finalize = wnck_pager_finalize;
 
   widget_class->realize = wnck_pager_realize;
@@ -238,6 +259,7 @@ wnck_pager_class_init (WnckPagerClass *klass)
   widget_class->drag_data_received = wnck_pager_drag_data_received;
   widget_class->drag_data_get = wnck_pager_drag_data_get;
   widget_class->drag_end = wnck_pager_drag_end;
+  widget_class->query_tooltip = wnck_pager_query_tooltip;
 }
 
 static void
@@ -247,8 +269,6 @@ wnck_pager_finalize (GObject *object)
 
   pager = WNCK_PAGER (object);
 
-  wnck_pager_disconnect_screen (pager);
-
   if (pager->priv->bg_cache)
     {
       g_object_unref (G_OBJECT (pager->priv->bg_cache));
@@ -260,10 +280,40 @@ wnck_pager_finalize (GObject *object)
       g_source_remove (pager->priv->dnd_activate);
       pager->priv->dnd_activate = 0;
     }
-
-  g_free (pager->priv);
   
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (wnck_pager_parent_class)->finalize (object);
+}
+
+static void
+_wnck_pager_set_screen (WnckPager *pager)
+{
+  GdkScreen *gdkscreen;
+
+  gdkscreen = gtk_widget_get_screen (GTK_WIDGET (pager));
+  pager->priv->screen = wnck_screen_get (gdk_screen_get_number (gdkscreen));
+
+  if (!wnck_pager_set_layout_hint (pager))
+    {
+      _WnckLayoutOrientation orientation;
+
+      /* we couldn't set the layout on the screen. This means someone else owns
+       * it. Let's at least show the correct layout. */
+      _wnck_screen_get_workspace_layout (pager->priv->screen,
+                                         &orientation,
+                                         &pager->priv->n_rows,
+                                         NULL, NULL);
+
+      /* test in this order to default to horizontal in case there was in issue
+       * when fetching the layout */
+      if (orientation == WNCK_LAYOUT_ORIENTATION_VERTICAL)
+        pager->priv->orientation = GTK_ORIENTATION_VERTICAL;
+      else
+        pager->priv->orientation = GTK_ORIENTATION_HORIZONTAL;
+
+      gtk_widget_queue_resize (GTK_WIDGET (pager));
+    }
+
+  wnck_pager_connect_screen (pager);
 }
 
 static void
@@ -276,6 +326,8 @@ wnck_pager_realize (GtkWidget *widget)
 
   pager = WNCK_PAGER (widget);
 
+  /* do not call the parent class realize since we're doing things a bit
+   * differently here */
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
 
   attributes.window_type = GDK_WINDOW_CHILD;
@@ -299,7 +351,11 @@ wnck_pager_realize (GtkWidget *widget)
   widget->style = gtk_style_attach (widget->style, widget->window);
   gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
 
-  wnck_pager_set_layout_hint (pager);
+  /* connect to the screen of this pager. In theory, this will already have
+   * been done in wnck_pager_size_request() */
+  if (pager->priv->screen == NULL)
+    _wnck_pager_set_screen (pager);
+  g_assert (pager->priv->screen != NULL);
 }
 
 static void
@@ -311,11 +367,16 @@ wnck_pager_unrealize (GtkWidget *widget)
 
   wnck_pager_clear_drag (pager);
   pager->priv->prelight = -1;
+  pager->priv->prelight_dnd = FALSE;
 
   wnck_screen_release_workspace_layout (pager->priv->screen,
                                         pager->priv->layout_manager_token);
+  pager->priv->layout_manager_token = WNCK_NO_MANAGER_TOKEN;
   
-  GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+  wnck_pager_disconnect_screen (pager);
+  pager->priv->screen = NULL;
+
+  GTK_WIDGET_CLASS (wnck_pager_parent_class)->unrealize (widget);
 }
 
 static void
@@ -334,6 +395,11 @@ wnck_pager_size_request  (GtkWidget      *widget,
 
   pager = WNCK_PAGER (widget);
   
+  /* if we're not realized, we don't know about our screen yet */
+  if (pager->priv->screen == NULL)
+    _wnck_pager_set_screen (pager);
+  g_assert (pager->priv->screen != NULL);
+
   n_spaces = wnck_screen_get_workspace_count (pager->priv->screen);
 
   g_assert (pager->priv->n_rows > 0);
@@ -491,7 +557,8 @@ wnck_pager_size_allocate (GtkWidget      *widget,
       return;
     }
 
-  GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+  GTK_WIDGET_CLASS (wnck_pager_parent_class)->size_allocate (widget,
+                                                             allocation);
 }
 
 static void
@@ -616,13 +683,14 @@ wnck_pager_window_state_is_relevant (int state)
 }
 
 static gint
-wnck_pager_window_get_workspace (WnckWindow *window)
+wnck_pager_window_get_workspace (WnckWindow *window,
+                                 gboolean    is_state_relevant)
 {
   gint state;
   WnckWorkspace *workspace;
 
   state = wnck_window_get_state (window);
-  if (!wnck_pager_window_state_is_relevant (state))
+  if (is_state_relevant && !wnck_pager_window_state_is_relevant (state))
     return -1;
   workspace = wnck_window_get_workspace (window);
   if (workspace == NULL && wnck_window_is_pinned (window))
@@ -647,7 +715,7 @@ get_windows_for_workspace_in_bottom_to_top (WnckScreen    *screen,
   for (tmp = windows; tmp != NULL; tmp = tmp->next)
     {
       WnckWindow *win = WNCK_WINDOW (tmp->data);
-      if (wnck_pager_window_get_workspace (win) == workspace_num)
+      if (wnck_pager_window_get_workspace (win, TRUE) == workspace_num)
 	result = g_list_prepend (result, win);
     }
 
@@ -796,6 +864,45 @@ draw_window (GdkDrawable        *drawable,
   cairo_destroy (cr);
 }            
 
+static WnckWindow *
+window_at_point (WnckPager     *pager,
+                 WnckWorkspace *space,
+                 GdkRectangle  *space_rect,
+                 int            x,
+                 int            y)
+{
+  WnckWindow *window;
+  GList *windows;
+  GList *tmp;
+
+  window = NULL;
+
+  windows = get_windows_for_workspace_in_bottom_to_top (pager->priv->screen,
+                                                        space);
+
+  /* clicks on top windows first */
+  windows = g_list_reverse (windows);
+
+  for (tmp = windows; tmp != NULL; tmp = tmp->next)
+    {
+      WnckWindow *win = WNCK_WINDOW (tmp->data);
+      GdkRectangle winrect;
+
+      get_window_rect (win, space_rect, &winrect);
+
+      if (POINT_IN_RECT (x, y, winrect))
+        {
+          /* wnck_window_activate (win); */
+          window = win;
+          break;
+        }
+    }
+
+  g_list_free (windows);
+
+  return window;
+}
+
 static int
 workspace_at_point (WnckPager *pager,
                     int        x,
@@ -940,7 +1047,7 @@ wnck_pager_draw_workspace (WnckPager    *pager,
     }
   else if (active_space)
     {
-      int vx, vy, vw, vh; /* viewport */
+      double vx, vy, vw, vh; /* viewport */
       double width_ratio, height_ratio;
       cairo_t *cr;
       
@@ -1023,7 +1130,7 @@ wnck_pager_draw_workspace (WnckPager    *pager,
       cairo_set_line_width (cr, 1.0);
       cairo_rectangle (cr,
 		       rect->x + 0.5, rect->y + 0.5,
-		       rect->width - 1, rect->height - 1);
+		       MAX (0, rect->width - 1), MAX (0, rect->height - 1));
       cairo_stroke (cr);
       cairo_destroy (cr);
     }
@@ -1143,33 +1250,11 @@ wnck_pager_button_press (GtkWidget      *widget,
     }
 
   if (space && (pager->priv->display_mode != WNCK_PAGER_DISPLAY_NAME))
-  {
-    GList *windows;
-    GList *tmp;
-
-    windows = get_windows_for_workspace_in_bottom_to_top (pager->priv->screen,
-                                                          space);
- 
-    /* clicks on top windows first */
-    windows = g_list_reverse (windows);
-
-    for (tmp = windows; tmp != NULL; tmp = tmp->next)
-      {
-        WnckWindow *win = WNCK_WINDOW (tmp->data);
-        GdkRectangle winrect;
-
-        get_window_rect (win, &workspace_rect, &winrect);
-
-        if (POINT_IN_RECT (event->x, event->y, winrect))
-          {
-            /* wnck_window_activate (win); */
-            pager->priv->drag_window = win;
-            break;
-          }
-      }
-
-    g_list_free (windows);
-  }
+    {
+      pager->priv->drag_window = window_at_point (pager, space,
+                                                  &workspace_rect,
+                                                  event->x, event->y);
+    }
 
   return TRUE;
 }
@@ -1213,7 +1298,7 @@ wnck_pager_queue_draw_window (WnckPager  *pager,
 {
   gint workspace;
 
-  workspace = wnck_pager_window_get_workspace (window);
+  workspace = wnck_pager_window_get_workspace (window, TRUE);
   if (workspace == -1)
     return;
 
@@ -1409,8 +1494,8 @@ wnck_pager_drag_motion_leave (GtkWidget          *widget,
 static void
 wnck_drag_clean_up (WnckWindow     *window,
 		    GdkDragContext *context,
-		    gboolean	    clean_up_window,
-		    gboolean	    clean_up_context);
+		    gboolean	    clean_up_for_context_destroy,
+		    gboolean	    clean_up_for_window_destroy);
 
 static void
 wnck_drag_context_destroyed (gpointer  windowp,
@@ -1430,6 +1515,9 @@ wnck_update_drag_icon (WnckWindow     *window,
   GtkWidget *widget;
 
   widget = g_object_get_data (G_OBJECT (context), "wnck-drag-source-widget");
+  if (!widget)
+    return;
+
   if (!gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
 					  GTK_ICON_SIZE_DND, &dnd_w, &dnd_h))
     dnd_w = dnd_h = 32;
@@ -1461,7 +1549,7 @@ wnck_update_drag_icon (WnckWindow     *window,
   gtk_drag_set_icon_pixmap (context, 
                             gdk_drawable_get_colormap (GDK_DRAWABLE (pixmap)),
 			    pixmap, NULL,
-			    rect.width / 4, 0);
+			    -2, -2);
 
   g_object_unref (pixmap);
 }
@@ -1474,16 +1562,31 @@ wnck_drag_window_destroyed (gpointer  contextp,
                       FALSE, TRUE);
 }
 
+static void
+wnck_drag_source_destroyed (gpointer  contextp,
+                            GObject  *drag_source)
+{
+  g_object_steal_data (G_OBJECT (contextp), "wnck-drag-source-widget");
+}
+
 /* CAREFUL: This function is a bit brittle, because the pointers given may be
  * finalized already */
 static void
 wnck_drag_clean_up (WnckWindow     *window,
 		    GdkDragContext *context,
-		    gboolean	    clean_up_window,
-		    gboolean	    clean_up_context)
+		    gboolean	    clean_up_for_context_destroy,
+		    gboolean	    clean_up_for_window_destroy)
 {
-  if (clean_up_window)
+  if (clean_up_for_context_destroy)
     {
+      GtkWidget *drag_source;
+
+      drag_source = g_object_get_data (G_OBJECT (context),
+                                       "wnck-drag-source-widget");
+      if (drag_source)
+        g_object_weak_unref (G_OBJECT (drag_source),
+                             wnck_drag_source_destroyed, context);
+
       g_object_weak_unref (G_OBJECT (window),
                            wnck_drag_window_destroyed, context);
       if (g_signal_handlers_disconnect_by_func (window,
@@ -1491,7 +1594,7 @@ wnck_drag_clean_up (WnckWindow     *window,
 	g_assert_not_reached ();
     }
 
-  if (clean_up_context)
+  if (clean_up_for_window_destroy)
     {
       g_object_steal_data (G_OBJECT (context), "wnck-drag-source-widget");
       g_object_weak_unref (G_OBJECT (context),
@@ -1503,7 +1606,8 @@ wnck_drag_clean_up (WnckWindow     *window,
  * wnck_window_set_as_drag_icon:
  * @window: #WnckWindow to use as drag icon
  * @context: #GdkDragContext to set the icon on
- * @drag_source: #GtkWidget that started the drag
+ * @drag_source: #GtkWidget that started the drag, or one of its parent. This
+ * widget needs to stay alive as long as possible during the drag.
  *
  * Sets the given @window as the drag icon for @context.
  **/
@@ -1522,6 +1626,8 @@ _wnck_window_set_as_drag_icon (WnckWindow     *window,
                     G_CALLBACK (wnck_update_drag_icon), context);
 
   g_object_set_data (G_OBJECT (context), "wnck-drag-source-widget", drag_source);
+  g_object_weak_ref (G_OBJECT (drag_source), wnck_drag_source_destroyed, context);
+
   g_object_weak_ref (G_OBJECT (context), wnck_drag_context_destroyed, window);
 
   wnck_update_drag_icon (window, context);
@@ -1643,22 +1749,88 @@ wnck_pager_focus (GtkWidget        *widget,
 
   pager = WNCK_PAGER (widget);
   
-  return GTK_WIDGET_CLASS (parent_class)->focus (widget, direction);
+  return GTK_WIDGET_CLASS (wnck_pager_parent_class)->focus (widget, direction);
 }
 
+/**
+ * wnck_pager_set_screen:
+ * @pager: a #WnckPager.
+ * @screen: a #WnckScreen.
+ *
+ * Does nothing.
+ *
+ * Since: 2.2
+ * Deprecated:
+ */
 void
 wnck_pager_set_screen (WnckPager  *pager,
 		       WnckScreen *screen)
 {
-  if (pager->priv->screen == screen)
-    return;
-
-  if (pager->priv->screen)
-    wnck_pager_disconnect_screen (pager);
-
-  wnck_pager_connect_screen (pager, screen);
 }
 
+static gboolean
+wnck_pager_query_tooltip (GtkWidget  *widget,
+                          gint        x,
+                          gint        y,
+                          gboolean    keyboard_tip,
+                          GtkTooltip *tooltip)
+{
+  int i;
+  WnckPager *pager;
+  WnckScreen *screen;
+  WnckWorkspace *space;
+  char *name;
+
+  pager = WNCK_PAGER (widget);
+  screen = pager->priv->screen;
+
+  i = workspace_at_point (pager, x, y, NULL, NULL);
+  space = wnck_screen_get_workspace (screen, i);
+  if (!space)
+    return GTK_WIDGET_CLASS (wnck_pager_parent_class)->query_tooltip (widget,
+                                                                      x, y,
+                                                                      keyboard_tip,
+                                                                      tooltip);
+
+  if (wnck_screen_get_active_workspace (screen) == space)
+    {
+      WnckWindow *window;
+      GdkRectangle workspace_rect;
+
+      get_workspace_rect (pager, i, &workspace_rect);
+
+      window = window_at_point (pager, space, &workspace_rect, x, y);
+
+      if (window)
+        name = g_strdup_printf (_("Click to start dragging \"%s\""),
+                                wnck_window_get_name (window));
+      else
+        name = g_strdup_printf (_("Current workspace: \"%s\""),
+                                wnck_workspace_get_name (space));
+    }
+  else
+    {
+      name = g_strdup_printf (_("Click to switch to \"%s\""),
+                              wnck_workspace_get_name (space));
+    }
+
+  gtk_tooltip_set_text (tooltip, name);
+
+  g_free (name);
+
+  return TRUE;
+}
+
+/**
+ * wnck_pager_new:
+ * @screen: deprecated argument, can be %NULL.
+ *
+ * Creates a new #WnckPager. The #WnckPager will show the #WnckWorkspace of the
+ * #WnckScreen it is on.
+ *
+ * Return value: a newly created #WnckPager.
+ */
+/* TODO: when we break API again, remove the screen from here */
 GtkWidget*
 wnck_pager_new (WnckScreen *screen)
 {
@@ -1666,24 +1838,27 @@ wnck_pager_new (WnckScreen *screen)
   
   pager = g_object_new (WNCK_TYPE_PAGER, NULL);
 
-  wnck_pager_connect_screen (pager, screen);
-
   return GTK_WIDGET (pager);
 }
 
-static void
+static gboolean
 wnck_pager_set_layout_hint (WnckPager *pager)
 {
   int layout_rows;
   int layout_cols;
 
+  /* if we're not realized, we don't know about our screen yet */
+  if (pager->priv->screen == NULL)
+    _wnck_pager_set_screen (pager);
+  g_assert (pager->priv->screen != NULL);
+
   /* The visual representation of the pager doesn't
    * correspond to the layout of the workspaces
    * here. i.e. the user will not pay any attention
    * to the n_rows setting on this pager.
    */
   if (!pager->priv->show_all_workspaces)
-    return;
+    return FALSE;
 
   if (pager->priv->orientation == GTK_ORIENTATION_HORIZONTAL)
     {
@@ -1701,39 +1876,115 @@ wnck_pager_set_layout_hint (WnckPager *pager)
                                           pager->priv->layout_manager_token,
                                           layout_rows,
                                           layout_cols);
+
+  return (pager->priv->layout_manager_token != WNCK_NO_MANAGER_TOKEN);
 }
 
-void
+/**
+ * wnck_pager_set_orientation:
+ * @pager: a #WnckPager.
+ * @orientation: orientation to use for the layout of #WnckWorkspace on the
+ * #WnckScreen @pager is watching.
+ *
+ * Tries to change the orientation of the layout of #WnckWorkspace on the
+ * #WnckScreen @pager is watching. Since no more than one application should
+ * set this property of a #WnckScreen at a time, setting the layout is not
+ * guaranteed to work. 
+ *
+ * If @orientation is %GTK_ORIENTATION_HORIZONTAL, the #WnckWorkspace will be
+ * laid out in rows, with the first #WnckWorkspace in the top left corner.
+ *
+ * If @orientation is %GTK_ORIENTATION_VERTICAL, the #WnckWorkspace will be
+ * laid out in columns, with the first #WnckWorkspace in the top left corner.
+ *
+ * For example, if the layout contains one row, but the orientation of the
+ * layout is vertical, the #WnckPager will display a column of #WnckWorkspace.
+ *
+ * Return value: %TRUE if the layout of #WnckWorkspace has been successfully
+ * changed or did not need to be changed, %FALSE otherwise.
+ */
+gboolean
 wnck_pager_set_orientation (WnckPager     *pager,
                             GtkOrientation orientation)
 {
-  g_return_if_fail (WNCK_IS_PAGER (pager));
+  GtkOrientation old_orientation;
+  gboolean       old_orientation_is_valid;
+
+  g_return_val_if_fail (WNCK_IS_PAGER (pager), FALSE);
 
   if (pager->priv->orientation == orientation)
-    return;
+    return TRUE;
+
+  old_orientation = pager->priv->orientation;
+  old_orientation_is_valid = pager->priv->screen != NULL;
 
   pager->priv->orientation = orientation;
-  gtk_widget_queue_resize (GTK_WIDGET (pager));
 
-  wnck_pager_set_layout_hint (pager);
+  if (wnck_pager_set_layout_hint (pager))
+    {
+      gtk_widget_queue_resize (GTK_WIDGET (pager));
+      return TRUE;
+    }
+  else
+    {
+      if (old_orientation_is_valid)
+        pager->priv->orientation = old_orientation;
+      return FALSE;
+    }
 }
 
-void
+/**
+ * wnck_pager_set_n_rows:
+ * @pager: a #WnckPager.
+ * @n_rows: the number of rows to use for the layout of #WnckWorkspace on the
+ * #WnckScreen @pager is watching.
+ *
+ * Tries to change the number of rows in the layout of #WnckWorkspace on the
+ * #WnckScreen @pager is watching. Since no more than one application should
+ * set this property of a #WnckScreen at a time, setting the layout is not
+ * guaranteed to work. 
+ *
+ * Return value: %TRUE if the layout of #WnckWorkspace has been successfully
+ * changed or did not need to be changed, %FALSE otherwise.
+ */
+gboolean
 wnck_pager_set_n_rows (WnckPager *pager,
 		       int        n_rows)
 {
-  g_return_if_fail (WNCK_IS_PAGER (pager));
-  g_return_if_fail (n_rows > 0);
+  int      old_n_rows;
+  gboolean old_n_rows_is_valid;
+
+  g_return_val_if_fail (WNCK_IS_PAGER (pager), FALSE);
+  g_return_val_if_fail (n_rows > 0, FALSE);
 
   if (pager->priv->n_rows == n_rows)
-    return;
+    return TRUE;
+
+  old_n_rows = pager->priv->n_rows;
+  old_n_rows_is_valid = pager->priv->screen != NULL;
 
   pager->priv->n_rows = n_rows;
-  gtk_widget_queue_resize (GTK_WIDGET (pager));
 
-  wnck_pager_set_layout_hint (pager);
+  if (wnck_pager_set_layout_hint (pager))
+    {
+      gtk_widget_queue_resize (GTK_WIDGET (pager));
+      return TRUE;
+    }
+  else
+    {
+      if (old_n_rows_is_valid)
+        pager->priv->n_rows = old_n_rows;
+      return FALSE;
+    }
 }
 
+/**
+ * wnck_pager_set_display_mode:
+ * @pager: a #WnckPager.
+ * @mode: a display mode.
+ *
+ * Sets the display mode for @pager to @mode.
+ */
 void
 wnck_pager_set_display_mode (WnckPager            *pager,
 			     WnckPagerDisplayMode  mode)
@@ -1743,10 +1994,20 @@ wnck_pager_set_display_mode (WnckPager            *pager,
   if (pager->priv->display_mode == mode)
     return;
 
+  g_object_set (pager, "has-tooltip", mode != WNCK_PAGER_DISPLAY_NAME, NULL);
+
   pager->priv->display_mode = mode;
   gtk_widget_queue_resize (GTK_WIDGET (pager));
 }
 
+/**
+ * wnck_pager_set_show_all:
+ * @pager: a #WnckPager.
+ * @show_all_workspaces: whether to display all #WnckWorkspace in @pager.
+ *
+ * Sets @pager to display all #WnckWorkspace or not, according to
+ * @show_all_workspaces.
+ */
 void
 wnck_pager_set_show_all (WnckPager *pager,
 			 gboolean   show_all_workspaces)
@@ -1762,6 +2023,17 @@ wnck_pager_set_show_all (WnckPager *pager,
   gtk_widget_queue_resize (GTK_WIDGET (pager));
 }
 
+/**
+ * wnck_pager_set_shadow_type:
+ * @pager: a #WnckPager.
+ * @shadow_type: a shadow type.
+ *
+ * Sets the shadow type for @pager to @shadow_type. The main use of this
+ * function is proper integration of #WnckPager in panels with non-system
+ * backgrounds.
+ *
+ * Since: 2.2
+ */
 void
 wnck_pager_set_shadow_type (WnckPager *   pager,
 			    GtkShadowType shadow_type)
@@ -1877,7 +2149,16 @@ window_state_changed_callback     (WnckWindow      *window,
                                    gpointer         data)
 {
   WnckPager *pager = WNCK_PAGER (data);
-  wnck_pager_queue_draw_window (pager, window);
+
+  /* if the changed state changes the visibility in the pager, we need to
+   * redraw the whole workspace. wnck_pager_queue_draw_window() might not be
+   * enough */
+  if (!wnck_pager_window_state_is_relevant (changed))
+    wnck_pager_queue_draw_workspace (pager,
+                                     wnck_pager_window_get_workspace (window,
+                                                                      FALSE));
+  else
+    wnck_pager_queue_draw_window (pager, window);
 }
 
 static void
@@ -1935,17 +2216,17 @@ viewports_changed_callback (WnckWorkspace *space,
 }
 
 static void
-wnck_pager_connect_screen (WnckPager  *pager,
-                           WnckScreen *screen)
+wnck_pager_connect_screen (WnckPager *pager)
 {
   int i;
   guint *c;
   GList *tmp;
+  WnckScreen *screen;
   
-  g_return_if_fail (pager->priv->screen == NULL);
-  
-  pager->priv->screen = screen;
+  g_return_if_fail (pager->priv->screen != NULL);
 
+  screen = pager->priv->screen;
+  
   for (tmp = wnck_screen_get_windows (screen); tmp; tmp = tmp->next)
     {
       wnck_pager_connect_window (pager, WNCK_WINDOW (tmp->data));
@@ -2068,8 +2349,6 @@ wnck_pager_disconnect_screen (WnckPager  *pager)
       space = wnck_screen_get_workspace (pager->priv->screen, i);
       g_signal_handlers_disconnect_by_func (space, G_CALLBACK (workspace_name_changed_callback), pager);
     }
-
-  pager->priv->screen = NULL;
 }
 
 static void
@@ -2185,7 +2464,7 @@ wnck_pager_get_accessible (GtkWidget *widget)
         }
       first_time = FALSE;
     }
-  return GTK_WIDGET_CLASS (parent_class)->get_accessible (widget);
+  return GTK_WIDGET_CLASS (wnck_pager_parent_class)->get_accessible (widget);
 }
 
 int 
