@@ -55,11 +55,17 @@ WnckWindow* _wnck_window_create  (Window      xwindow,
                                   gint        sort_order);
 void        _wnck_window_destroy (WnckWindow *window);
 
+char*       _wnck_window_get_name_for_display (WnckWindow *window,
+                                               gboolean    use_icon_name,
+                                               gboolean    use_state_decorations);
 const char* _wnck_window_get_startup_id (WnckWindow *window);
 
 const char* _wnck_window_get_resource_class (WnckWindow *window);
 const char* _wnck_window_get_resource_name  (WnckWindow *window);
 
+time_t      _wnck_window_get_needs_attention_time (WnckWindow *window);
+time_t      _wnck_window_or_transient_get_needs_attention_time (WnckWindow *window);
+
 WnckWorkspace* _wnck_workspace_create  (int            number,
 					WnckScreen    *screen);
 void           _wnck_workspace_destroy (WnckWorkspace *space);
@@ -85,6 +91,14 @@ WnckApplication* _wnck_application_create  (Window           xwindow,
                                             WnckScreen      *screen);
 void             _wnck_application_destroy (WnckApplication *app);
 
+
+WnckClassGroup*  _wnck_class_group_create        (const char     *res_class);
+void             _wnck_class_group_destroy       (WnckClassGroup *class_group);
+void             _wnck_class_group_add_window    (WnckClassGroup *class_group,
+                                                  WnckWindow     *window);
+void             _wnck_class_group_remove_window (WnckClassGroup *class_group,
+                                                  WnckWindow     *window);
+
 void _wnck_workspace_update_name (WnckWorkspace *workspace,
                                   const char    *name);
 void _wnck_screen_change_workspace_name (WnckScreen *screen,
@@ -126,6 +140,7 @@ void           _wnck_pager_get_workspace_rect   (WnckPager     *pager,
 
 void           _make_gtk_label_bold   (GtkLabel *label);
 void           _make_gtk_label_normal (GtkLabel *label);
+
 void           _wnck_stock_icons_init (void);
 
 
