@@ -127,6 +127,7 @@ struct _WnckScreenPrivate
 };
 
 G_DEFINE_TYPE (WnckScreen, wnck_screen, G_TYPE_OBJECT);
+#define WNCK_SCREEN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), WNCK_TYPE_SCREEN, WnckScreenPrivate))
 
 enum {
   ACTIVE_WINDOW_CHANGED,
@@ -194,8 +195,50 @@ static guint signals[LAST_SIGNAL] = { 0 };
 static void
 wnck_screen_init (WnckScreen *screen)
 {  
-  screen->priv = g_new0 (WnckScreenPrivate, 1);
+  screen->priv = WNCK_SCREEN_GET_PRIVATE (screen);
 
+  screen->priv->number = -1;
+  screen->priv->xroot = None;
+  screen->priv->xscreen = NULL;
+
+  screen->priv->mapped_windows = NULL;
+  screen->priv->stacked_windows = NULL;
+  screen->priv->workspaces = NULL;
+
+  screen->priv->active_window = NULL;
+  screen->priv->previously_active_window = NULL;
+
+  screen->priv->active_workspace = NULL;
+
+  screen->priv->window_order = 0;
+
+  screen->priv->bg_pixmap = None;
+
+  screen->priv->wm_name = NULL;
+
+  screen->priv->update_handler = 0;
+
+#ifdef HAVE_STARTUP_NOTIFICATION
+  screen->priv->sn_display = NULL;
+#endif
+
+  screen->priv->showing_desktop = FALSE;
+
+  screen->priv->vertical_workspaces = FALSE;
+  screen->priv->starting_corner = WNCK_LAYOUT_CORNER_TOPLEFT;
+  screen->priv->rows_of_workspaces = 1;
+  screen->priv->columns_of_workspaces = -1;
+
+  screen->priv->need_update_stack_list = FALSE;
+  screen->priv->need_update_workspace_list = FALSE;
+  screen->priv->need_update_viewport_settings = FALSE;
+  screen->priv->need_update_active_workspace = FALSE;
+  screen->priv->need_update_active_window = FALSE;
+  screen->priv->need_update_workspace_layout = FALSE;
+  screen->priv->need_update_workspace_names = FALSE;
+  screen->priv->need_update_bg_pixmap = FALSE;
+  screen->priv->need_update_showing_desktop = FALSE;
+  screen->priv->need_update_wm = FALSE;
 }
 
 static void
@@ -204,7 +247,9 @@ wnck_screen_class_init (WnckScreenClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   _wnck_init ();
-  
+
+  g_type_class_add_private (klass, sizeof (WnckScreenPrivate));
+
   object_class->finalize = wnck_screen_finalize;
 
   /**
@@ -460,19 +505,33 @@ static void
 wnck_screen_finalize (GObject *object)
 {
   WnckScreen *screen;
+  GList *tmp;
   gpointer weak_pointer;
 
   screen = WNCK_SCREEN (object);
   
   unqueue_update (screen);
 
-  /* FIXME this isn't right, we need to destroy the items
-   * in the list. But it doesn't matter at the
-   * moment since we never finalize screens anyway.
-   */
+  for (tmp = screen->priv->stacked_windows; tmp; tmp = tmp->next)
+    {
+      screen->priv->mapped_windows = g_list_remove (screen->priv->mapped_windows,
+                                                    tmp->data);
+      _wnck_window_destroy (WNCK_WINDOW (tmp->data));
+    }
+
+  for (tmp = screen->priv->mapped_windows; tmp; tmp = tmp->next)
+    _wnck_window_destroy (WNCK_WINDOW (tmp->data));
+
+  for (tmp = screen->priv->workspaces; tmp; tmp = tmp->next)
+    g_object_unref (tmp->data);
+
   g_list_free (screen->priv->mapped_windows);
+  screen->priv->mapped_windows = NULL;
   g_list_free (screen->priv->stacked_windows);
+  screen->priv->stacked_windows = NULL;
+
   g_list_free (screen->priv->workspaces);
+  screen->priv->workspaces = NULL;
 
   weak_pointer = &screen->priv->active_window;
   if (screen->priv->active_window != NULL)
@@ -487,15 +546,15 @@ wnck_screen_finalize (GObject *object)
   screen->priv->previously_active_window = NULL;
 
   g_free (screen->priv->wm_name);
+  screen->priv->wm_name = NULL;
 
   screens[screen->priv->number] = NULL;
 
 #ifdef HAVE_STARTUP_NOTIFICATION
   sn_display_unref (screen->priv->sn_display);
+  screen->priv->sn_display = NULL;
 #endif
   
-  g_free (screen->priv);
-  
   G_OBJECT_CLASS (wnck_screen_parent_class)->finalize (object);
 }
 
@@ -523,12 +582,6 @@ wnck_screen_construct (WnckScreen *screen,
   screen->priv->xroot = RootWindow (gdk_display, number);
   screen->priv->xscreen = ScreenOfDisplay (gdk_display, number);
   screen->priv->number = number;
-  screen->priv->window_order = 0;
-
-  screen->priv->rows_of_workspaces = 1;
-  screen->priv->columns_of_workspaces = -1;
-  screen->priv->vertical_workspaces = FALSE;
-  screen->priv->starting_corner = WNCK_LAYOUT_CORNER_TOPLEFT;
 
 #ifdef HAVE_STARTUP_NOTIFICATION
   screen->priv->sn_display = sn_display_new (gdk_display,
@@ -561,14 +614,17 @@ wnck_screen_construct (WnckScreen *screen,
  * 
  * Returns the #WnckScreen for a given screen on the default display.
  * 
- * Return value: the #WnckScreen for screen @index. The returned #WnckScreen is
- * owned by libwnck and must not be referenced or unreferenced.
+ * Return value: the #WnckScreen for screen @index, or %NULL if no such screen
+ * exists. The returned #WnckScreen is owned by libwnck and must not be
+ * referenced or unreferenced.
  **/
 WnckScreen*
 wnck_screen_get (int index)
 {
   g_return_val_if_fail (gdk_display != NULL, NULL);
-  g_return_val_if_fail (index < ScreenCount (gdk_display), NULL);
+
+  if (index >= ScreenCount (gdk_display))
+    return NULL;
   
   if (screens == NULL)
     {
@@ -669,6 +725,26 @@ wnck_screen_get_number (WnckScreen *screen)
   return screen->priv->number;
 }
 
+/**
+ * wnck_screen_get_workspaces:
+ * @screen: a #WnckScreen.
+ * 
+ * Returns the list of #WnckWorkspace on @screen. The list is ordered: the
+ * first element in the list is the first #WnckWorkspace, etc..
+ * 
+ * Return value: the list of #WnckWorkspace on @screen. The list should not be
+ * modified nor freed, as it is owned by @screen.
+ *
+ * Since: 2.20
+ **/
+GList*
+wnck_screen_get_workspaces (WnckScreen *screen)
+{
+  g_return_val_if_fail (WNCK_IS_SCREEN (screen), NULL);
+  
+  return screen->priv->workspaces;
+}
+
 /**
  * wnck_screen_get_workspace:
  * @screen: a #WnckScreen.
@@ -710,8 +786,8 @@ wnck_screen_get_workspace (WnckScreen *screen,
  * Return value: the index of @space on @screen, or -1 on errors.
  *
  * Since: 2.14
+ * Deprecated: Use wnck_workspace_get_number() instead.
  **/
-/* TODO: when we break API again, kill this function */
 int
 wnck_screen_get_workspace_index (WnckScreen    *screen,
                                  WnckWorkspace *space)
@@ -770,6 +846,7 @@ wnck_screen_get_active_workspace (WnckScreen *screen)
  * or unreferenced.
  *
  * Since: 2.14
+ * Deprecated: Use wnck_workspace_get_neighbor() instead.
  **/
 WnckWorkspace*
 wnck_screen_get_workspace_neighbor (WnckScreen         *screen,
@@ -864,7 +941,7 @@ wnck_screen_get_previously_active_window (WnckScreen *screen)
  * wnck_screen_get_windows:
  * @screen: a #WnckScreen.
  * 
- * Returns the list of #WnckWindow on #WnckScreen. The list is not in a defined
+ * Returns the list of #WnckWindow on @screen. The list is not in a defined
  * order, but should be "stable" (windows should not be reordered in it).
  * However, the stability of the list is established by the window manager, so
  * don't blame libwnck if it breaks down.
@@ -885,7 +962,7 @@ wnck_screen_get_windows (WnckScreen *screen)
  * wnck_screen_get_windows_stacked:
  * @screen: a #WnckScreen.
  * 
- * Returns the list of #WnckWindow on #WnckScreen in bottom-to-top order.
+ * Returns the list of #WnckWindow on @screen in bottom-to-top order.
  * 
  * Return value: the list of #WnckWindow in stacking order on @screen, or %NULL
  * if there is no window on @screen. The list should not be modified nor freed,
@@ -982,11 +1059,13 @@ wnck_screen_change_workspace_count (WnckScreen *screen,
   xev.xclient.format = 32;
   xev.xclient.data.l[0] = count;
   
+  _wnck_error_trap_push ();
   XSendEvent (DisplayOfScreen (screen->priv->xscreen),
               screen->priv->xroot,
               False,
               SubstructureRedirectMask | SubstructureNotifyMask,
               &xev);
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -1077,9 +1156,10 @@ _wnck_screen_process_property_notify (WnckScreen *screen,
  * the row and column of the #WnckWorkspace with index @space_index.
  *
  * Since: 2.12
+ * Deprecated:
  */
-/* TODO: when we break API again, make this function (and WnckWorkspaceLayout)
- * private. Also remove num_workspaces since we can get it from screen! */
+/* TODO: when we make this private, remove num_workspaces since we can get it
+ * from screen! */
 void
 wnck_screen_calc_workspace_layout (WnckScreen          *screen,
                                    int                  num_workspaces,
@@ -1295,9 +1375,8 @@ wnck_screen_calc_workspace_layout (WnckScreen          *screen,
  * might want to free @layout yourself after calling this.
  *
  * Since: 2.12
+ * Deprecated:
  */
-/* TODO: when we break API again, make this function (and WnckWorkspaceLayout)
- * private. */
 void
 wnck_screen_free_workspace_layout (WnckWorkspaceLayout *layout)
 {
@@ -2055,15 +2134,23 @@ update_workspace_layout (WnckScreen *screen)
             }
           else
             {
+              int num_workspaces;
+
+              num_workspaces = wnck_screen_get_workspace_count (screen);
+
               if (rows > 0)
                 screen->priv->rows_of_workspaces = rows;
               else
-                screen->priv->rows_of_workspaces = -1;
+                screen->priv->rows_of_workspaces =
+                                      num_workspaces / cols
+                                      + ((num_workspaces % cols) > 0 ? 1 : 0);
 
               if (cols > 0)
                 screen->priv->columns_of_workspaces = cols;
               else
-                screen->priv->columns_of_workspaces = -1;
+                screen->priv->columns_of_workspaces =
+                                      num_workspaces / rows
+                                      + ((num_workspaces % rows) > 0 ? 1 : 0);
             }
           if (n_items == 4)
             {
