@@ -85,6 +85,8 @@ static void object_weak_notify (gpointer data,
 static void window_weak_notify (gpointer data,
                                 GObject *window);
 
+static void refill_submenu (ActionMenuData *amd);
+
 static void
 window_weak_notify (gpointer data,
                     GObject *window)
@@ -179,36 +181,32 @@ item_activated_callback (GtkWidget *menu_item,
     case LEFT:
       {
         WnckWorkspace *workspace;
-        workspace = wnck_screen_get_workspace_neighbor (wnck_window_get_screen (amd->window),
-                                                        wnck_window_get_workspace (amd->window), 
-                                                        WNCK_MOTION_LEFT);
+        workspace = wnck_workspace_get_neighbor (wnck_window_get_workspace (amd->window), 
+                                                 WNCK_MOTION_LEFT);
         wnck_window_move_to_workspace (amd->window, workspace);
         break;
       }
     case RIGHT:
       {
         WnckWorkspace *workspace;
-        workspace = wnck_screen_get_workspace_neighbor (wnck_window_get_screen ( amd->window),
-                                                        wnck_window_get_workspace (amd->window),
-                                                        WNCK_MOTION_RIGHT);
+        workspace = wnck_workspace_get_neighbor (wnck_window_get_workspace (amd->window), 
+                                                 WNCK_MOTION_RIGHT);
         wnck_window_move_to_workspace (amd->window, workspace);
         break;
       }
     case UP:
       {
         WnckWorkspace *workspace;
-        workspace = wnck_screen_get_workspace_neighbor (wnck_window_get_screen (amd->window),
-                                                        wnck_window_get_workspace (amd->window),
-                                                        WNCK_MOTION_UP);
+        workspace = wnck_workspace_get_neighbor (wnck_window_get_workspace (amd->window), 
+                                                 WNCK_MOTION_UP);
         wnck_window_move_to_workspace (amd->window, workspace);
         break;
       }
     case DOWN:
       {
         WnckWorkspace *workspace;
-        workspace = wnck_screen_get_workspace_neighbor (wnck_window_get_screen (amd->window),
-                                                        wnck_window_get_workspace (amd->window),
-                                                        WNCK_MOTION_DOWN);
+        workspace = wnck_workspace_get_neighbor (wnck_window_get_workspace (amd->window), 
+                                                 WNCK_MOTION_DOWN);
         wnck_window_move_to_workspace (amd->window, workspace);
         break;
       }
@@ -219,8 +217,9 @@ item_activated_callback (GtkWidget *menu_item,
         workspace_index = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menu_item), "workspace"));
 
         wnck_window_move_to_workspace (amd->window,
-                                       wnck_screen_get_workspace (wnck_window_get_screen (amd->window),
-                                       workspace_index));
+                                       wnck_screen_get_workspace (
+                                           wnck_window_get_screen (amd->window),
+                                           workspace_index));
         break;
       }
     }
@@ -268,6 +267,7 @@ update_menu_state (ActionMenuData *amd)
 {
   WnckWindowActions  actions;  
   WnckScreen        *screen;
+  WnckWorkspace     *workspace;
 
   amd->idle_handler = 0;
   
@@ -350,43 +350,69 @@ update_menu_state (ActionMenuData *amd)
   gtk_widget_set_sensitive (amd->resize_item,
                             (actions & WNCK_WINDOW_ACTION_RESIZE) != 0);
 
-  if (!wnck_window_is_pinned (amd->window))
+  gtk_widget_set_sensitive (amd->workspace_item,
+                            (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+
+  gtk_widget_set_sensitive (amd->left_item,
+                            (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+  gtk_widget_set_sensitive (amd->right_item,
+                            (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+  gtk_widget_set_sensitive (amd->up_item,
+                            (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+  gtk_widget_set_sensitive (amd->down_item,
+                            (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+
+  workspace = wnck_window_get_workspace (amd->window);
+
+  if (workspace && !wnck_window_is_pinned (amd->window))
     {
-      if (amd->workspace_item)
-        gtk_widget_set_sensitive (amd->workspace_item,
-                                  (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+      if (wnck_workspace_get_neighbor (workspace, WNCK_MOTION_LEFT))
+        gtk_widget_show (amd->left_item);
+      else
+        gtk_widget_hide (amd->left_item);
 
-      if (amd->left_item)
-        gtk_widget_set_sensitive (amd->left_item,
-                                  (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+      if (wnck_workspace_get_neighbor (workspace, WNCK_MOTION_RIGHT))
+        gtk_widget_show (amd->right_item);
+      else
+        gtk_widget_hide (amd->right_item);
 
-      if (amd->right_item)
-        gtk_widget_set_sensitive (amd->right_item,
-                                  (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+      if (wnck_workspace_get_neighbor (workspace, WNCK_MOTION_UP))
+        gtk_widget_show (amd->up_item);
+      else
+        gtk_widget_hide (amd->up_item);
 
-      if (amd->up_item)
-        gtk_widget_set_sensitive (amd->up_item,
-                                  (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
-
-      if (amd->down_item)
-        gtk_widget_set_sensitive (amd->down_item,
-                                  (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+      if (wnck_workspace_get_neighbor (workspace, WNCK_MOTION_DOWN))
+        gtk_widget_show (amd->down_item);
+      else
+        gtk_widget_hide (amd->down_item);
     }
+  else
+    {
+      gtk_widget_hide (amd->left_item);
+      gtk_widget_hide (amd->right_item);
+      gtk_widget_hide (amd->up_item);
+      gtk_widget_hide (amd->down_item);
+    }
+
   if (wnck_screen_get_workspace_count (screen) > 1)
     {
       gtk_widget_show (amd->workspace_separator);
       gtk_widget_show (amd->pin_item);
       gtk_widget_show (amd->unpin_item);
       gtk_widget_show (amd->workspace_item);
+      refill_submenu (amd);
     }
   else
     {
       gtk_widget_hide (amd->workspace_separator);
       gtk_widget_hide (amd->pin_item);
-      gtk_widget_show (amd->unpin_item);
+      gtk_widget_hide (amd->unpin_item);
       gtk_widget_hide (amd->workspace_item);
+      gtk_menu_popdown (GTK_MENU (gtk_menu_item_get_submenu (GTK_MENU_ITEM (amd->workspace_item))));
     }
-  
+
+  gtk_menu_reposition (GTK_MENU (amd->menu));
+
   return FALSE;
 }
 
@@ -425,6 +451,31 @@ actions_changed_callback (WnckWindow       *window,
     queue_update (amd);
 }
 
+static void
+workspace_changed_callback (WnckWindow *window,
+                            gpointer    data)
+{
+  ActionMenuData *amd;
+
+  amd = get_data (data);
+
+  if (amd)
+    queue_update (amd);
+}
+
+static void
+screen_workspace_callback (WnckWindow    *window,
+                           WnckWorkspace *space,
+                           gpointer       data)
+{
+  ActionMenuData *amd;
+
+  amd = get_data (data);
+
+  if (amd)
+    queue_update (amd);
+}
+
 static GtkWidget*
 make_radio_menu_item (ActionMenuData  *amd,
                       WindowAction     action,
@@ -486,7 +537,7 @@ amd_free (ActionMenuData *amd)
   if (amd->idle_handler)
     g_source_remove (amd->idle_handler);
 
-  g_free (amd);
+  g_slice_free (ActionMenuData, amd);
 }
 
 static char *
@@ -561,6 +612,57 @@ get_workspace_name_with_accel (WnckWindow *window,
   }
 }
 
+static void
+refill_submenu (ActionMenuData *amd)
+{
+  GtkWidget *submenu;
+  GList *children;
+  GList *l;
+  int num_workspaces, window_space, i;
+  WnckWorkspace *workspace;
+
+  submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (amd->workspace_item));
+
+  /* Remove existing items */
+  children = gtk_container_get_children (GTK_CONTAINER (submenu));
+  for (l = children; l; l = l->next)
+    gtk_container_remove (GTK_CONTAINER (submenu), l->data);
+  g_list_free (children);
+
+  workspace = wnck_window_get_workspace (amd->window);
+
+  num_workspaces = wnck_screen_get_workspace_count (wnck_window_get_screen (amd->window));
+
+  if (workspace)
+    window_space = wnck_workspace_get_number (workspace);
+  else
+    window_space = -1;
+
+  for (i = 0; i < num_workspaces; i++)
+    {
+      char *name, *label;
+      GtkWidget *item;
+	
+      name = get_workspace_name_with_accel (amd->window, i);
+      label = g_strdup_printf ("%s", name);
+
+      item = make_menu_item (amd, MOVE_TO_WORKSPACE);
+      g_object_set_data (G_OBJECT (item), "workspace", GINT_TO_POINTER (i));
+
+      if (i == window_space)
+        gtk_widget_set_sensitive (item, FALSE);
+
+      gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
+      set_item_text (item, label);
+      set_item_stock (item, NULL);
+
+      g_free (name);
+      g_free (label);	
+    }
+
+  gtk_menu_reposition (GTK_MENU (submenu));
+}
+
 /**
  * wnck_create_window_action_menu:
  * @window: the #WnckWindow for which a menu will be created.
@@ -576,16 +678,14 @@ wnck_create_window_action_menu (WnckWindow *window)
   GtkWidget *menu, *submenu;
   ActionMenuData *amd;
   GtkWidget *separator;
-  int num_workspaces, window_space, i;
-  WnckWorkspace *workspace;
-  WnckWorkspaceLayout layout;
   GSList *pin_group;
+  WnckScreen *screen;
 
   g_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
 
   _wnck_stock_icons_init ();
   
-  amd = g_new0 (ActionMenuData, 1);
+  amd = g_slice_new0 (ActionMenuData);
   amd->window = window;
   
   menu = gtk_menu_new ();
@@ -647,67 +747,29 @@ wnck_create_window_action_menu (WnckWindow *window)
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          amd->pin_item);
   
-  workspace = wnck_window_get_workspace (amd->window);
+  amd->left_item = make_menu_item (amd, LEFT);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         amd->left_item);
+  set_item_text (amd->left_item, _("Move to Workspace _Left"));
+  set_item_stock (amd->left_item, NULL);
 
-  num_workspaces = wnck_screen_get_workspace_count (wnck_window_get_screen (amd->window));
+  amd->right_item = make_menu_item (amd, RIGHT);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         amd->right_item);
+  set_item_text (amd->right_item, _("Move to Workspace R_ight"));
+  set_item_stock (amd->right_item, NULL);
 
-  if (workspace)
-    window_space = wnck_workspace_get_number (workspace);
-  else
-    window_space = -1; //FIXME: this probably causes weird results. How to verify?
+  amd->up_item = make_menu_item (amd, UP);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         amd->up_item);
+  set_item_text (amd->up_item, _("Move to Workspace _Up"));
+  set_item_stock (amd->up_item, NULL);
 
-  wnck_screen_calc_workspace_layout (wnck_window_get_screen (amd->window),
-                                     num_workspaces,
-                                     window_space,
-                                     &layout);
-
-  if (!wnck_window_is_pinned (amd->window))
-    {
-      if (layout.current_col > 0)
-        {
-          amd->left_item = make_menu_item (amd, LEFT);
-          gtk_menu_shell_append (GTK_MENU_SHELL (menu),
-                                 amd->left_item);
-          set_item_text (amd->left_item, _("Move to Workspace _Left"));
-          set_item_stock (amd->left_item, NULL);
-        }
-      else 
-         amd->left_item = NULL;
-
-      if ((layout.current_col < layout.cols - 1) && (layout.current_row * layout.cols + (layout.current_col + 1) < num_workspaces ))
-        {
-          amd->right_item = make_menu_item (amd, RIGHT);
-          gtk_menu_shell_append (GTK_MENU_SHELL (menu),
-                                 amd->right_item);
-          set_item_text (amd->right_item, _("Move to Workspace R_ight"));
-          set_item_stock (amd->right_item, NULL);
-        }
-      else 
-         amd->right_item = NULL;       
-       
-      if (layout.current_row > 0)
-        {
-          amd->up_item = make_menu_item (amd, UP);
-          gtk_menu_shell_append (GTK_MENU_SHELL (menu),
-                                 amd->up_item);
-          set_item_text (amd->up_item, _("Move to Workspace _Up"));
-          set_item_stock (amd->up_item, NULL);
-        }
-      else 
-         amd->up_item = NULL; 
-
-
-      if ((layout.current_row < layout.rows - 1) && ((layout.current_row + 1) * layout.cols + layout.current_col) < num_workspaces)
-        {
-          amd->down_item = make_menu_item (amd, DOWN);
-          gtk_menu_shell_append (GTK_MENU_SHELL (menu),
-                                 amd->down_item);
-          set_item_text (amd->down_item, _("Move to Workspace _Down"));
-          set_item_stock (amd->down_item, NULL);
-        }
-      else 
-         amd->down_item = NULL;  
-    }
+  amd->down_item = make_menu_item (amd, DOWN);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         amd->down_item);
+  set_item_text (amd->down_item, _("Move to Workspace _Down"));
+  set_item_stock (amd->down_item, NULL);
 
   amd->workspace_item = gtk_menu_item_new_with_mnemonic (_("Move to Another _Workspace")); 
   gtk_widget_show (amd->workspace_item);
@@ -719,28 +781,6 @@ wnck_create_window_action_menu (WnckWindow *window)
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          amd->workspace_item);
 
-  for (i = 0; i < num_workspaces; i++)
-    {
-      char *name, *label;
-      GtkWidget *item;
-	
-      name = get_workspace_name_with_accel (amd->window, i);
-      label = g_strdup_printf ("%s", name);
-
-      item = make_menu_item (amd, MOVE_TO_WORKSPACE);
-      g_object_set_data (G_OBJECT (item), "workspace", GINT_TO_POINTER (i));
-
-      if (i == window_space)
-        gtk_widget_set_sensitive (item, FALSE);
-
-      gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
-      set_item_text (item, label);
-      set_item_stock (item, NULL);
-
-      g_free (name);
-      g_free (label);	
-    }
-
   separator = gtk_separator_menu_item_new ();
   gtk_widget_show (separator);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
@@ -766,8 +806,26 @@ wnck_create_window_action_menu (WnckWindow *window)
                            G_OBJECT (menu),
                            0);
 
+  g_signal_connect_object (G_OBJECT (amd->window), 
+                           "workspace_changed",
+                           G_CALLBACK (workspace_changed_callback),
+                           G_OBJECT (menu),
+                           0);
+
+  screen = wnck_window_get_screen (amd->window);
+
+  g_signal_connect_object (G_OBJECT (screen), 
+                           "workspace_created",
+                           G_CALLBACK (screen_workspace_callback),
+                           G_OBJECT (menu),
+                           0);
+  g_signal_connect_object (G_OBJECT (screen), 
+                           "workspace_destroyed",
+                           G_CALLBACK (screen_workspace_callback),
+                           G_OBJECT (menu),
+                           0);
+
   update_menu_state (amd);
-  wnck_screen_free_workspace_layout (&layout);
   
   return menu;
 }
