@@ -1,4 +1,5 @@
 /* Xlib utils */
+/* vim: set sw=2 et: */
 
 /*
  * Copyright (C) 2001 Havoc Pennington
@@ -849,11 +850,13 @@ _wnck_close (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
               RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev); 
+  _wnck_error_trap_pop ();
 }
 
 #define _NET_WM_MOVERESIZE_SIZE_TOPLEFT      0
@@ -887,11 +890,13 @@ _wnck_keyboard_move (Screen *screen,
   xev.xclient.data.l[3] = 0; /* unused */
   xev.xclient.data.l[4] = _wnck_get_client_type ();
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
               RootWindowOfScreen (screen),
               False,
               SubstructureRedirectMask | SubstructureNotifyMask,
               &xev); 
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -913,11 +918,13 @@ _wnck_keyboard_size (Screen *screen,
   xev.xclient.data.l[3] = 0; /* unused */
   xev.xclient.data.l[4] = _wnck_get_client_type ();
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
               RootWindowOfScreen (screen),
               False,
               SubstructureRedirectMask | SubstructureNotifyMask,
               &xev); 
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -946,11 +953,13 @@ _wnck_change_state (Screen  *screen,
   xev.xclient.data.l[3] = _wnck_get_client_type ();
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -973,11 +982,13 @@ _wnck_change_workspace (Screen     *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -1004,11 +1015,13 @@ _wnck_activate (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev); 
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -1031,11 +1044,13 @@ _wnck_activate_workspace (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -1058,11 +1073,13 @@ _wnck_change_viewport (Screen *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
+  _wnck_error_trap_pop ();
 }
 
 void
@@ -1084,11 +1101,13 @@ _wnck_toggle_showing_desktop (Screen  *screen,
   xev.xclient.data.l[3] = 0;
   xev.xclient.data.l[4] = 0;
 
+  _wnck_error_trap_push ();
   XSendEvent (DisplayOfScreen (screen),
 	      RootWindowOfScreen (screen),
               False,
 	      SubstructureRedirectMask | SubstructureNotifyMask,
 	      &xev);
+  _wnck_error_trap_pop ();
 }
 
 char*
@@ -1140,6 +1159,25 @@ _wnck_get_name (Window xwindow)
   return name;
 }
 
+char*
+_wnck_get_icon_name (Window xwindow)
+{
+  char *name;
+  
+  name = _wnck_get_utf8_property (xwindow,
+                                  _wnck_atom_get ("_NET_WM_VISIBLE_ICON_NAME"));
+
+  if (name == NULL)
+    name = _wnck_get_utf8_property (xwindow,
+                                    _wnck_atom_get ("_NET_WM_ICON_NAME"));
+
+  if (name == NULL)
+    name = _wnck_get_text_property (xwindow,
+                                    XA_WM_ICON_NAME);
+
+  return name;
+}
+
 static char*
 latin1_to_utf8 (const char *latin1)
 {
@@ -1211,6 +1249,41 @@ _wnck_get_wmclass (Window xwindow,
     }
 }
 
+gboolean
+_wnck_get_frame_extents (Window  xwindow,
+                         int    *left_frame,
+                         int    *right_frame,
+                         int    *top_frame,
+                         int    *bottom_frame)
+{
+  gulong   *p_size;
+  int       n_size;
+  gboolean  retval;
+
+  retval = FALSE;
+  p_size = NULL;
+  n_size = 0;
+
+  _wnck_get_cardinal_list (xwindow,
+                           _wnck_atom_get ("_NET_FRAME_EXTENTS"),
+                           &p_size, &n_size);
+
+  if (p_size != NULL && n_size == 4)
+    {
+      *left_frame   = p_size[0];
+      *right_frame  = p_size[1];
+      *top_frame    = p_size[2];
+      *bottom_frame = p_size[3];
+
+      retval = TRUE;
+    }
+
+  if (p_size != NULL)
+    g_free (p_size);
+
+  return retval;
+}
+
 void
 _wnck_select_input (Window xwindow,
                     int    mask)
@@ -1593,7 +1666,10 @@ get_cmap (GdkPixmap *pixmap)
   if (cmap &&
       (gdk_colormap_get_visual (cmap)->depth !=
        gdk_drawable_get_depth (pixmap)))
-    cmap = NULL;
+    {
+      g_object_unref (G_OBJECT (cmap));
+      cmap = NULL;
+    }
   
   return cmap;
 }
@@ -1624,22 +1700,22 @@ _wnck_gdk_pixbuf_get_from_pixmap (GdkPixbuf   *dest,
 
   if (drawable)
     {
-			cmap = get_cmap (drawable);
+      cmap = get_cmap (drawable);
 
-			/* GDK is supposed to do this but doesn't in GTK 2.0.2,
-			 * fixed in 2.0.3
-			 */
-			if (width < 0)
-				gdk_drawable_get_size (drawable, &width, NULL);
-			if (height < 0)
-				gdk_drawable_get_size (drawable, NULL, &height);
-			
-			retval = gdk_pixbuf_get_from_drawable (dest,
-																						 drawable,
-																						 cmap,
-																						 src_x, src_y,
-																						 dest_x, dest_y,
-																						 width, height);
+      /* GDK is supposed to do this but doesn't in GTK 2.0.2,
+       * fixed in 2.0.3
+       */
+      if (width < 0)
+        gdk_drawable_get_size (drawable, &width, NULL);
+      if (height < 0)
+        gdk_drawable_get_size (drawable, NULL, &height);
+
+      retval = gdk_pixbuf_get_from_drawable (dest,
+                                             drawable,
+                                             cmap,
+                                             src_x, src_y,
+                                             dest_x, dest_y,
+                                             width, height);
     }
 
   if (cmap)
@@ -1803,7 +1879,7 @@ _wnck_icon_cache_new (void)
 {
   WnckIconCache *icon_cache;
 
-  icon_cache = g_new0 (WnckIconCache, 1);
+  icon_cache = g_slice_new0 (WnckIconCache);
 
   icon_cache->origin = USING_NO_ICON;
   icon_cache->prev_pixmap = None;
@@ -1848,7 +1924,7 @@ _wnck_icon_cache_free (WnckIconCache *icon_cache)
 {
   clear_icon_cache (icon_cache, FALSE);
   
-  g_free (icon_cache);
+  g_slice_free (WnckIconCache, icon_cache);
 }
 
 void
@@ -2263,11 +2339,13 @@ void _wnck_set_window_geometry (Screen *screen,
   xev.xclient.data.l[3] = width;
   xev.xclient.data.l[4] = height;
 
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display,
               RootWindowOfScreen (screen),
               False,
               SubstructureRedirectMask | SubstructureNotifyMask,
               &xev);
+  _wnck_error_trap_push ();
 }
 
 void
@@ -2321,24 +2399,6 @@ _wnck_set_icon_geometry  (Window xwindow,
   _wnck_error_trap_pop ();
 }
 
-void
-_wnck_set_dock_type_hint (Window xwindow)
-{
-  Atom atom;
-
-  atom = _wnck_atom_get ("_NET_WM_WINDOW_TYPE_DOCK");
-  
-  _wnck_error_trap_push ();
-
-  XChangeProperty (gdk_display,
-                   xwindow, 
-                   _wnck_atom_get ("_NET_WM_WINDOW_TYPE"),
-		   XA_ATOM, 32, PropModeReplace,
-		   (guchar *)&atom, 1);
-
-  _wnck_error_trap_pop ();
-}
-
 /* orientation of pager */
 #define _NET_WM_ORIENTATION_HORZ 0
 #define _NET_WM_ORIENTATION_VERT 1
@@ -2413,21 +2473,20 @@ timestamp_predicate (Display *display,
  * Return value: the time stamp.
  **/
 static Time
-get_server_time (Display *display,
-		 Window   window)
+get_server_time (Window window)
 {
   unsigned char c = 'a';
   XEvent xevent;
   TimeStampInfo info;
 
-  info.timestamp_prop_atom = XInternAtom  (display, "_TIMESTAMP_PROP", False);
+  info.timestamp_prop_atom = _wnck_atom_get ("_TIMESTAMP_PROP");
   info.window = window;
 
-  XChangeProperty (display, window,
+  XChangeProperty (gdk_display, window,
 		   info.timestamp_prop_atom, info.timestamp_prop_atom,
 		   8, PropModeReplace, &c, 1);
 
-  XIfEvent (display, &xevent,
+  XIfEvent (gdk_display, &xevent,
 	    timestamp_predicate, (XPointer)&info);
 
   return xevent.xproperty.time;
@@ -2445,19 +2504,38 @@ typedef struct
 static GSList *layout_managers = NULL;
 static int next_token = 1;
 
+static void
+_wnck_free_layout_manager (LayoutManager *lm)
+{
+  _wnck_error_trap_push ();
+  XDestroyWindow (gdk_display, lm->window);
+  _wnck_error_trap_pop ();
+
+  g_slice_free (LayoutManager, lm);
+
+  layout_managers = g_slist_remove (layout_managers, lm);
+}
+
 int
 _wnck_try_desktop_layout_manager (Screen *xscreen,
                                   int     current_token)
 {
+  Atom selection_atom;
+  Window owner;
   GSList *tmp;
   int number;
   Time timestamp;
   XClientMessageEvent xev;  
   char buffer[256];
   LayoutManager *lm;
-  
+
   number = XScreenNumberOfScreen (xscreen);
   
+  sprintf (buffer, "_NET_DESKTOP_LAYOUT_S%d", number);
+  selection_atom = _wnck_atom_get (buffer);
+
+  owner = XGetSelectionOwner (gdk_display, selection_atom);
+  
   tmp = layout_managers;
   while (tmp != NULL)
     {
@@ -2466,7 +2544,15 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
       if (number == lm->screen_number)
         {
           if (current_token == lm->token)
-            return current_token; /* we still have the selection */
+            {
+              if (owner == lm->window)
+                return current_token; /* we still have the selection */
+              else
+                { /* we lost the selection */
+                  _wnck_free_layout_manager (lm);
+                  break;
+                }
+            }
           else
             return WNCK_NO_MANAGER_TOKEN; /* someone else has it */
         }
@@ -2474,17 +2560,21 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
       tmp = tmp->next;
     }
   
-  /* No one in-process has the selection at the moment */
+  if (owner != None)
+    return WNCK_NO_MANAGER_TOKEN; /* someone else has the selection */
 
-  lm = g_new0 (LayoutManager, 1);
+  /* No one has the selection at the moment */
+
+  lm = g_slice_new0 (LayoutManager);
 
   lm->screen_number = number;
   lm->token = next_token;
   ++next_token;
 
-  sprintf (buffer, "_NET_DESKTOP_LAYOUT_S%d", number);
-  lm->selection_atom = XInternAtom (gdk_display, buffer, False);
-  lm->manager_atom = XInternAtom (gdk_display, "MANAGER", False);
+  lm->selection_atom = selection_atom;
+  lm->manager_atom = _wnck_atom_get ("MANAGER");
+
+  _wnck_error_trap_push ();
 
   lm->window = XCreateSimpleWindow (gdk_display,
                                     RootWindowOfScreen (xscreen),
@@ -2493,11 +2583,13 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
                                     WhitePixel (gdk_display, number));
 
   XSelectInput (gdk_display, lm->window, PropertyChangeMask);
-  timestamp = get_server_time (gdk_display, lm->window);
+  timestamp = get_server_time (lm->window);
 
   XSetSelectionOwner (gdk_display, lm->selection_atom,
 		      lm->window, timestamp);
 
+  _wnck_error_trap_pop ();
+
   /* Check to see if we managed to claim the selection. */
 
   if (XGetSelectionOwner (gdk_display, lm->selection_atom) !=
@@ -2517,8 +2609,10 @@ _wnck_try_desktop_layout_manager (Screen *xscreen,
   xev.data.l[3] = 0;	/* manager specific data */
   xev.data.l[4] = 0;	/* manager specific data */
   
+  _wnck_error_trap_push ();
   XSendEvent (gdk_display, RootWindow (gdk_display, number),
               False, StructureNotifyMask, (XEvent *)&xev);
+  _wnck_error_trap_pop ();
 
   layout_managers = g_slist_prepend (layout_managers,
                                      lm);
@@ -2545,12 +2639,22 @@ _wnck_release_desktop_layout_manager (Screen *xscreen,
         {
           if (current_token == lm->token)
             {
-              XDestroyWindow (gdk_display,
-                              lm->window);
+              _wnck_error_trap_push ();
 
-              g_free (lm);
-              layout_managers = g_slist_remove (layout_managers, lm);
-              
+              /* release selection ownership */
+              if (XGetSelectionOwner (gdk_display, lm->selection_atom) !=
+                  lm->window)
+                {
+                  Time timestamp;
+
+                  timestamp = get_server_time (lm->window);
+                  XSetSelectionOwner (gdk_display, lm->selection_atom,
+                                      None, timestamp);
+                }
+
+              _wnck_error_trap_pop ();
+
+              _wnck_free_layout_manager (lm);
               return;
             }
         }
@@ -2576,12 +2680,7 @@ _wnck_desktop_layout_manager_process_event (XEvent *xev)
       if (xev->xany.window == lm->window &&
           xev->xselectionclear.selection == lm->selection_atom)
         {
-          XDestroyWindow (gdk_display,
-                          lm->window);
-          
-          g_free (lm);
-          layout_managers = g_slist_remove (layout_managers, lm);
-          
+          _wnck_free_layout_manager (lm);
           return TRUE;
         }
       
@@ -2590,51 +2689,3 @@ _wnck_desktop_layout_manager_process_event (XEvent *xev)
 
   return FALSE;
 }
-
-/* stock icon code Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org> */
-typedef struct
-{
-  char *stock_id;
-  const guint8 *icon_data;
-} StockIcon;
-
-void
-_wnck_stock_icons_init (void)
-{
-  GtkIconFactory *factory;
-  int i;
-  static gboolean done = FALSE;
-
-  StockIcon items[] =
-  {
-    { WNCK_STOCK_DELETE,   stock_delete_data   },
-    { WNCK_STOCK_MINIMIZE, stock_minimize_data },
-    { WNCK_STOCK_MAXIMIZE, stock_maximize_data }
-  };
-
-  if (done)
-    return;
-
-  done = TRUE;
-  
-  factory = gtk_icon_factory_new ();
-  gtk_icon_factory_add_default (factory);
-
-  for (i = 0; i < (gint) G_N_ELEMENTS (items); i++)
-    {
-      GtkIconSet *icon_set;
-      GdkPixbuf *pixbuf;
-
-      pixbuf = gdk_pixbuf_new_from_inline (-1, items[i].icon_data,
-					   FALSE,
-					   NULL);
-
-      icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
-      gtk_icon_factory_add (factory, items[i].stock_id, icon_set);
-      gtk_icon_set_unref (icon_set);
-		
-      g_object_unref (G_OBJECT (pixbuf));
-    }
-
-  g_object_unref (G_OBJECT (factory));
-}
