@@ -1,4 +1,5 @@
 /* Xlib utilities */
+/* vim: set sw=2 et: */
 
 /*
  * Copyright (C) 2001 Havoc Pennington
@@ -112,10 +113,16 @@ void _wnck_change_viewport (Screen *screen,
 char*  _wnck_get_session_id     (Window xwindow);
 int    _wnck_get_pid            (Window xwindow);
 char*  _wnck_get_name           (Window xwindow);
+char*  _wnck_get_icon_name      (Window xwindow);
 char*  _wnck_get_res_class_utf8 (Window xwindow);
 void   _wnck_get_wmclass        (Window xwindow,
                                  char **res_class,
                                  char **res_name);
+gboolean _wnck_get_frame_extents  (Window  xwindow,
+                                   int    *left_frame,
+                                   int    *right_frame,
+                                   int    *top_frame,
+                                   int    *bottom_frame);
 
 void   _wnck_select_input     (Window xwindow,
                                int    mask);
@@ -182,8 +189,6 @@ void _wnck_set_icon_geometry  (Window xwindow,
 			       int    width,
 			       int    height);
 
-void _wnck_set_dock_type_hint (Window xwindow);
-
 void _wnck_set_desktop_layout (Screen *xscreen,
                                int     rows,
                                int     columns);
@@ -205,8 +210,6 @@ void     _wnck_release_desktop_layout_manager       (Screen *xscreen,
                                                      int     current_token);
 gboolean _wnck_desktop_layout_manager_process_event (XEvent *xev);
 
-void _wnck_stock_icons_init (void);
-
 G_END_DECLS
 
 #endif /* WNCK_XUTILS_H */
